@@ -25,6 +25,7 @@ void
 bitstream_close(bitstream_t * bs) {
     if (bs->data) {
         free(bs->data);
+        bs->data = NULL;
     }
     free(bs);
     return ;
@@ -75,13 +76,14 @@ unsigned char *
 bitstream_steal(bitstream_t *bs, unsigned long *length) {
     unsigned char *data, *tmp;
     *length = 0;
-    if (! bs) {
+    if (bs == NULL) {
+        fprintf(stderr, "bitstream_steal: bs == NULL\n");
         return NULL;
     }
     data = bs->data;
     *length = bs->data_len;
     if ((tmp = realloc(data, *length)) == NULL) {
-        fprintf(stderr, "Can't realloc\n");
+        fprintf(stderr, "bitstream_steal: Can't realloc\n");
     }
     bs->data = NULL;
     bs->data_len = 0;
@@ -92,8 +94,8 @@ bitstream_steal(bitstream_t *bs, unsigned long *length) {
 unsigned char *
 bitstream_output_sub(bitstream_t *bs, unsigned long offset, unsigned long length) {
     unsigned char *data;
-    if (! bs) {
-        fprintf(stderr, "bs == NULL");
+    if (bs == NULL) {
+        fprintf(stderr, "bitstream_output_sub: bs == NULL\n");
         return NULL;
     }
     if (bs->data_len < offset + length ) {
@@ -250,7 +252,7 @@ bitstream_putbit(bitstream_t *bs, int bit) {
 //        fprintf(stderr, "bs->data_len(%ld) <= bs->byte_offset(%ld)\n",
 //                bs->data_len, bs->byte_offset);
         if (bs->data_alloc_len <= bs->byte_offset) {
-            fprintf(stderr, "bitstream_putbit: alloc_len=%lu\n", bs->data_alloc_len);
+//            fprintf(stderr, "bitstream_putbit: alloc_len=%lu\n", bs->data_alloc_len);
             bitstream_realloc(bs);
         }
         bs->data[bs->byte_offset] = 0;
@@ -383,12 +385,11 @@ unsigned long
 bitstream_length(bitstream_t *bs) {
     return bs->data_len;
 }
+
 /*
  * utility
  */
 
-
-
 signed long
 bitstream_unsigned2signed(unsigned long num, int size) {
     unsigned long sig_bit = 1 << (size - 1);
@@ -439,7 +440,6 @@ bitstream_need_bits_signed(signed long n) {
     return ret;
 }
 
-
 /*
  * error handling
  */
@@ -454,6 +454,7 @@ void
 bitstream_printerror(bitstream_t *bs) {
     (void) bs;
 }
+
 /*
  * for debug
  */
