@@ -37,6 +37,7 @@
 #include "swf_tag_action.h"
 #include "swf_tag_sprite.h"
 #include "swf_tag_shape.h"
+#include "swf_tag_place.h"
 #include "swf_tag.h"
 #include "swf_object.h"
 
@@ -55,6 +56,7 @@ zend_function_entry swfed_functions[] = {
     PHP_ME(swfed,  __construct, NULL, 0)
     PHP_ME(swfed,  input, NULL, 0)
     PHP_ME(swfed,  output, NULL, 0)
+    PHP_ME(swfed,  _destroy_and_exit, NULL, 0) // for debug
     PHP_ME(swfed,  getHeaderInfo, NULL, 0)
     PHP_ME(swfed,  setHeaderInfo, NULL, 0)
     PHP_ME(swfed,  getTagList, NULL, 0)
@@ -84,12 +86,13 @@ zend_function_entry swfed_functions[] = {
     PHP_ME(swfed,  getActionData, NULL, 0)
     PHP_ME(swfed,  disasmActionData, NULL, 0)
     PHP_ME(swfed,  setActionVariables, NULL, 0)
+    PHP_ME(swfed,  replaceMovieClip, NULL, 0)
     PHP_ME(swfed,  swfInfo, NULL, 0)
     {NULL, NULL, NULL}	/* Must be the last line in swfed_functions[] */
 };
 /* }}} */
 
-#define SWFED_VERSION "0.31"
+#define SWFED_VERSION "0.34"
 
 /* {{{ swfed_module_entry
  */
@@ -310,6 +313,13 @@ PHP_METHOD(swfed, output) {
     RETURN_STRINGL(new_buff, len, 0);
 }
 
+PHP_METHOD(swfed, _destroy_and_exit) { // for debug
+    swf_object_t *swf = NULL;
+    swf = get_swf_object(getThis() TSRMLS_CC);
+    swf_object_close(swf);
+    exit(0);
+}
+
 PHP_METHOD(swfed, getHeaderInfo) {
     swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
     array_init(return_value);
@@ -406,7 +416,9 @@ PHP_METHOD(swfed, getTagDetail) {
     if ((tag_info == NULL) || (tag_info->detail_handler == NULL)) {
         RETURN_FALSE;
     }
-    swf_tag_create_input_detail(tag, swf);
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        RETURN_FALSE; // can't create detail
+    }
     switch (tag->tag) {
         swf_tag_jpeg_detail_t     *tag_jpeg;
         swf_tag_lossless_detail_t *tag_lossless;
@@ -415,6 +427,7 @@ PHP_METHOD(swfed, getTagDetail) {
         swf_tag_action_detail_t   *tag_action;
         swf_tag_sprite_detail_t   *tag_sprite;
         swf_tag_shape_detail_t    *tag_shape;
+        swf_tag_place_detail_t    *tag_place;
       case 6:  // DefineBitsJPEG
       case 21: // DefineBitsJPEG2
       case 35: // DefineBitsJPEG3
@@ -493,7 +506,21 @@ PHP_METHOD(swfed, getTagDetail) {
         add_assoc_long(return_value, "line_styles.count", tag_shape->shape_with_style.styles.line_styles.count);
 //        tag_shape->shape_with_style.shape_records
         break;
+      case 4: // PlaceObject
+      case 26: // PlaceObject2
+        tag_place = tag->detail;
+        array_init(return_value);
+        if ((tag->tag == 4) || tag_place->flag_has_character) {
+            add_assoc_long(return_value, "character_id", tag_place->character_id);
+        }
+        add_assoc_long(return_value, "depth", tag_place->depth);
+        if (tag_place->flag_has_name) {
+            add_assoc_string_ex(return_value, "name",
+                                sizeof("name"), (char *)tag_place->name, 1);
+        }
+
       default:
+          break;
         RETURN_FALSE;
     }
     /* return_value */
@@ -546,7 +573,6 @@ PHP_METHOD(swfed, getTagContentsByCID) {
     long cid = 0;
     swf_object_t *swf = NULL;
     unsigned char *data_ref = NULL;
-    char *new_buff = NULL;
     unsigned long data_len = 0;
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &cid) == FAILURE) {
         RETURN_FALSE;
@@ -591,7 +617,6 @@ PHP_METHOD(swfed, getShapeData) {
     long cid = 0;
     swf_object_t *swf = NULL;
     unsigned char *data_ref = NULL;
-    char *new_buff = NULL;
     unsigned long data_len = 0;
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &cid) == FAILURE) {
         RETURN_FALSE;
@@ -657,7 +682,7 @@ PHP_METHOD(swfed, getShapeIdListByBitmapRef) {
     i = 0;
     for (tag = swf->tag ; tag ; tag=tag->next) {
         register int tag_code = tag->tag;
-        if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_identity(tag, bitmap_id) == 0)) {
+        if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_get_refcid(tag) == bitmap_id)) {
             swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
             add_index_long(return_value, i, (long) swf_tag_shape->shape_id);
             i++;
@@ -1107,6 +1132,26 @@ PHP_METHOD(swfed, setActionVariables) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, replaceMovieClip) {
+    char *instance_name = NULL, *swf_data = NULL;
+    int  instance_name_len = 0, swf_data_len = 0;
+    swf_object_t *swf = NULL;
+    int result = 0;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss",
+                              &instance_name, &instance_name_len,
+                              &swf_data, &swf_data_len) == FAILURE) {
+        RETURN_FALSE;
+    }
+    swf = get_swf_object(getThis() TSRMLS_CC);    
+    result = swf_object_replace_movieclip(swf, instance_name,
+                                          instance_name_len,
+                                          swf_data, swf_data_len);
+    if (result) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
+
 PHP_METHOD(swfed, swfInfo) {
     swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
     swf_object_print(swf);
