@@ -97,6 +97,12 @@ extern unsigned char *swf_object_get_actiondata(swf_object_t *swf, unsigned long
 extern int swf_object_insert_action_setvariables(swf_object_t *swf,
                                                  y_keyvalue_t *kv);
 
+extern int swf_object_replace_movieclip(swf_object_t *swf,
+                                        unsigned char *instance_name,
+                                        int instancee_name_len,
+                                        unsigned char *swf_data,
+                                        int swf_data_len);
+
 
 extern int swf_object_apply_shapematrix_factor(swf_object_t *swf,int shape_id,
                                                double scale_x, double scale_y,
