@@ -16,6 +16,7 @@
 #include "swf_tag_sound.h"
 #include "swf_tag_sprite.h"
 #include "swf_tag_shape.h"
+#include "swf_tag_place.h"
 #include "bitmap_util.h"
 
 swf_tag_info_t swf_tag_info_table[] = {
@@ -23,7 +24,7 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 1, "ShowFrame", NULL },
     { 2, "DefineShape", swf_tag_shape_detail_handler },
     { 3, "FreeCharacter", NULL},
-    { 4, "PlaceObject", NULL},
+    { 4, "PlaceObject", swf_tag_place_detail_handler },
     { 5, "RemoveObject", NULL},
     { 6, "DefineBitsJPEG", swf_tag_jpeg_detail_handler },
     { 7, "DefineButton", NULL},
@@ -42,7 +43,7 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 20, "DefineBitsLossless", swf_tag_lossless_detail_handler },
     { 21, "DefineBitsJPEG2", swf_tag_jpeg_detail_handler },
     { 22, "DefineShape2", swf_tag_shape_detail_handler },
-    { 26, "PlaceObject2", NULL },
+    { 26, "PlaceObject2", swf_tag_place_detail_handler },
     { 28, "RemoveObject2", NULL },
     { 32, "DefineShape3", swf_tag_shape_detail_handler },
     { 33, "DefineText2", NULL },
@@ -77,7 +78,7 @@ swf_tag_info_t *get_swf_tag_info(int tag_id) {
 }
 
 swf_tag_t *swf_tag_create(bitstream_t *bs) {
-    swf_tag_t *tag = calloc(sizeof(*tag), 1);
+    swf_tag_t *tag = NULL;
     unsigned short tag_and_length;
     if (bs == NULL) {
         fprintf(stderr, "swf_tag_create: bs == NULL\n");
@@ -86,9 +87,9 @@ swf_tag_t *swf_tag_create(bitstream_t *bs) {
     tag_and_length = bitstream_getbytesLE(bs, 2);
     if (tag_and_length == (unsigned short) -1) {
         fprintf(stderr, "swf_tag_create: tag_and_length(short) == -1\n");
-        free(tag);
         return NULL;
     }
+    tag = calloc(sizeof(*tag), 1);
     tag->tag = tag_and_length >> 6;
     tag->length = tag_and_length & 0x3f;
     tag->length_longformat = 0;
@@ -109,7 +110,7 @@ swf_tag_t *swf_tag_create(bitstream_t *bs) {
 }
 
 void swf_tag_destroy(swf_tag_t *tag) {
-    if (! tag) {
+    if (tag == NULL) {
         return;
     }
     if (tag->data) {
@@ -117,29 +118,22 @@ void swf_tag_destroy(swf_tag_t *tag) {
         tag->data = NULL;
     }
     if (tag->detail) {
-        swf_tag_info_t *tag_info = get_swf_tag_info(tag->tag);
-        if (tag_info && tag_info->detail_handler) {
-            swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-            if (detail_handler->destroy) {
-               detail_handler->destroy(tag);
-            } else {
-                fprintf(stderr, "detail_handler->destroy == NULL (tag=%d)\n",
-                        tag->tag);
-            }
-        } else {
-            fprintf(stderr, "not impremented yet. destroy tag detail\n");
-        }
+        swf_tag_destroy_detail(tag);
+	tag->detail = NULL;
     }
     free(tag);
 }
 
-
 static int swf_tag_and_length_build(bitstream_t *bs, swf_tag_t *tag) {
     signed short tag_and_length;
     if (bs == NULL) {
         fprintf(stderr, "swf_tag_and_length_build: bs == NULL\n");
         return 1;
     }
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_and_length_build: tag == NULL\n");
+        return 1;
+    }
     if (tag->length >= 0x3f) {
         tag->length_longformat = 1;
     } else {
@@ -169,12 +163,16 @@ static int swf_tag_and_length_build(bitstream_t *bs, swf_tag_t *tag) {
 }
 
 int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *swf) {
-    swf_tag_info_t *tag_info;
-    unsigned char *data;
+    swf_tag_info_t *tag_info = NULL;
+    unsigned char *data = NULL;
     unsigned long data_len = 0;
-    swf_tag_detail_handler_t * detail_handler;
+    swf_tag_detail_handler_t *detail_handler = NULL;
     if (bs == NULL) {
-        fprintf(stderr, "swf_tag_and_length_build: bs == NULL\n");
+        fprintf(stderr, "swf_tag_build: bs == NULL\n");
+        return 1;
+    }
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_build: tag == NULL\n");
         return 1;
     }
 //    fprintf(stderr, "XXX: swf_tag_build: tag->tag=%d\n",tag->tag);
@@ -212,8 +210,8 @@ int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *swf) {
 
 void
 swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf, int indent_depth) {
-    swf_tag_info_t *tag_info;
-    const char *tag_name;
+    swf_tag_info_t *tag_info = NULL;
+    const char *tag_name = NULL;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_print: tag == NULL\n");
         return ;
@@ -227,8 +225,9 @@ swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf, int indent_depth) {
     }
     printf("\n");
     if (tag_info && tag_info->detail_handler) {
-        if (tag->detail == NULL) {
-            swf_tag_create_input_detail(tag, swf);
+        if (swf_tag_create_input_detail(tag, swf) == NULL) {
+            fprintf(stderr, "swf_tag_print: swf_tag_create_input_detail failed\n");
+            return ;
         }
         swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
         if (detail_handler->print) {
@@ -237,11 +236,14 @@ swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf, int indent_depth) {
     }
 }
 
-int swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf) {
-    swf_tag_info_t *tag_info;
+void *swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf) {
+    swf_tag_info_t *tag_info = NULL;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_create_input_detail: tag == NULL\n");
-        return 1;
+        return NULL;
+    }
+    if (tag->detail) {
+        return tag->detail;
     }
     tag_info = get_swf_tag_info(tag->tag);
     if (tag_info && tag_info->detail_handler) {
@@ -250,37 +252,163 @@ int swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf) {
         if (detail_handler->create == NULL) {
             fprintf(stderr, "detail_handler->create == NULL (tag=%d)\n",
                     tag->tag);
-            return 1;
+            return NULL;
         }
         tag->detail = detail_handler->create();
         if (tag->detail == NULL) {
             fprintf(stderr, "can't create tag detail (tag=%d)\n", tag->tag);
-            return 1;
+            return NULL;
         }
         result = detail_handler->input(tag, swf);
         if (result) {
             fprintf(stderr, "can't input tag detail (result=%d)\n", result);
-            return 1;
+            return NULL;
         }
     }
-    return 1;
+    return tag->detail;
+}
+
+void swf_tag_destroy_detail(swf_tag_t *tag) {
+    if (tag == NULL) {
+        return;
+    }
+    if (tag->detail) {
+        swf_tag_info_t *tag_info = get_swf_tag_info(tag->tag);
+        if (tag_info && tag_info->detail_handler) {
+            swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+            if (detail_handler->destroy) {
+                detail_handler->destroy(tag);
+            } else {
+	        fprintf(stderr, "detail_handler->destroy == NULL (tag=%d)\n",
+                        tag->tag);
+            }
+        } else {
+	    fprintf(stderr, "not impremented yet. destroy tag detail\n");
+        }
+	tag->detail = NULL;
+    }
+}
+
+
+int
+swf_tag_get_cid(swf_tag_t *tag) {
+    swf_tag_info_t *tag_info = NULL;
+    int tag_no;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_get_cid: tag == NULL\n");
+        return 1;
+    }
+    tag_no = tag->tag;
+    tag_info = get_swf_tag_info(tag_no);
+    if (tag_info && tag_info->detail_handler) {
+        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+        if (detail_handler->get_cid) {
+            return detail_handler->get_cid(tag);
+        }
+    } else {
+        switch (tag_no) {
+          case 7:  // DefineButton
+          case 10: // DefineFont
+          case 11: // DefineText
+          case 13: // DefineFontInfo
+          case 14: // DefineSound
+          case 17: // DefineButtonSound
+          case 33: // DefineText2
+          case 34: // DefineButton2
+          case 39: // DefineSprite ///////////////
+          case 46: // DefineMorphShape
+          case 48: // DefineFont2
+          case 88: // DefineFontName
+              if (tag->data) {
+                  return GetUShortLE(tag->data);
+              }
+              break;
+          default:
+              ;
+        }
+    }
+    return -1; // no cid tag
 }
 
 int
-swf_tag_identity(swf_tag_t *tag, int cid) {
+swf_tag_replace_cid(swf_tag_t *tag, int cid) {
     swf_tag_info_t *tag_info;
-    tag_info = get_swf_tag_info(tag->tag);
+    int tag_no;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_replace_cid: tag == NULL\n");
+        return 1; // failure
+    }
+    tag_no = tag->tag;
+    tag_info = get_swf_tag_info(tag_no);
     if (tag_info && tag_info->detail_handler) {
         swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-        if (detail_handler->identity) {
-            if (detail_handler->identity(tag, cid)) {
-                return 1; // no match
-            } else{
-                return 0; // match
+        if (detail_handler->replace_cid) {
+            return detail_handler->replace_cid(tag, cid);
+        }
+    } else {
+        switch (tag_no) {
+          case 7:  // DefineButton
+          case 10: // DefineFont
+          case 11: // DefineText
+          case 13: // DefineFontInfo
+          case 14: // DefineSound
+          case 17: // DefineButtonSound
+          case 33: // DefineText2
+          case 34: // DefineButton2
+          case 39: // DefineSprite ///////////////
+          case 46:  // DefineMorphShape
+          case 48: // DefineFont2
+          case 88: // DefineFontName
+            if (tag->data) {
+                PutUShortLE(tag->data, cid);
+                return 0;
             }
+            break;
+          default:
+              ;
         }
     }
-    return 1; // no match - no identity method
+    return 1; // no cid tag
+}
+
+int
+swf_tag_get_refcid(swf_tag_t *tag) {
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_get_refcid: tag == NULL\n");
+        return -1;
+    }
+    if (isPlaceTag(tag->tag)) { // PlaceObject, PlaceObject2
+        swf_tag_place_detail_t *swf_tag_place;
+        swf_tag_place = swf_tag_create_input_detail(tag, NULL);
+        if (swf_tag_place == NULL) {
+            fprintf(stderr, "swf_tag_get_refcid: swf_tag_place swf_tag_create_input_detail failed\n");
+            return -1;
+        }
+        return swf_tag_place->character_id;
+    }
+    return -1; // no cid tag
+}
+
+int
+swf_tag_replace_refcid(swf_tag_t *tag, int cid) {
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_replace_refcid: tag == NULL\n");
+        return 1; // failure
+    }
+    if (isPlaceTag(tag->tag)) { // PlaceObject, PlaceObject2
+        swf_tag_place_detail_t *swf_tag_place;
+        swf_tag_place = swf_tag_create_input_detail(tag, NULL);
+        if (swf_tag_place == NULL) {
+            fprintf(stderr, "swf_tag_replace_refcid: swf_tag_place swf_tag_create_input_detail failed\n");
+            return 1; // failure
+        }
+        swf_tag_place->character_id = cid;
+    }
+    if (tag->data) {
+        free(tag->data);
+        tag->data = NULL;
+    }
+    return 0; // success
 }
 
 /* bitmap */
@@ -293,11 +421,14 @@ swf_tag_get_bitmap_size(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_get_bitmap_size: tag == NULL\n");
         return 1;
     }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_get_bitmap_size: tag->detail == NULL\n");
+    if ((width == NULL ) || (height == NULL)) {
+        fprintf(stderr, "swf_tag_get_bitmap_size: width == NULL or height == NULL\n");
+        return 1;
+    }
+    if (swf_tag_create_input_detail(tag, NULL) == NULL) {
+        fprintf(stderr, "swf_tag_get_bitmap_size: swf_tag_create_input_detail failed\n");
         return 1;
     }
-
     if (isBitsJPEGTag(tag->tag)) {
         swf_tag_jpeg_detail_t *swf_tag_jpeg = (swf_tag_jpeg_detail_t *) tag->detail;
         ret = jpeg_size(swf_tag_jpeg->jpeg_data, swf_tag_jpeg->jpeg_data_len,
@@ -306,29 +437,30 @@ swf_tag_get_bitmap_size(swf_tag_t *tag,
         swf_tag_lossless_detail_t *swf_tag_lossless = (swf_tag_lossless_detail_t *) tag->detail;
         *width  = swf_tag_lossless->width;
         *height = swf_tag_lossless->height;
+    } else { // no Bitmap Tag
+        return 1;
     }
     return ret;
-
 }
 
-
 unsigned char *
 swf_tag_get_jpeg_data(swf_tag_t *tag, unsigned long *length, int image_id, swf_tag_t *tag_jpegtables) {
-    swf_tag_info_t *tag_info;
+    swf_tag_info_t *tag_info = NULL;
     *length = 0;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_get_jpeg_data: tag == NULL\n");
         return NULL;
     }
+    if (length == NULL) {
+        fprintf(stderr, "swf_tag_get_jpeg_data: length == NULL\n");
+        return NULL;
+    }
     tag_info = get_swf_tag_info(tag->tag);
     if ((tag->tag != 6) && (tag->tag != 21) && (tag->tag != 35)) {
         return NULL;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, NULL);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_get_jpeg_data: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, NULL) == NULL) {
+        fprintf(stderr, "swf_tag_get_jpeg_data: swf_tag_create_input_detail failed\n");
         return NULL;
     }
     if (tag_jpegtables) {
@@ -343,29 +475,23 @@ swf_tag_get_jpeg_data(swf_tag_t *tag, unsigned long *length, int image_id, swf_t
 
 unsigned char *
 swf_tag_get_alpha_data(swf_tag_t *tag, unsigned long *length, int image_id) {
-    swf_tag_info_t *tag_info;
     *length = 0;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_get_alpha_data: tag == NULL\n");
         return NULL;
     }
+    if (length == NULL) {
+        fprintf(stderr, "swf_tag_get_alpha_data: length == NULL\n");
+        return NULL;
+    }
     if (tag->tag != 35) { // ! DefineBitsJPEG3
         return NULL;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag_info && tag_info->detail_handler) {
-        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-        if (detail_handler->identity) {
-            if (detail_handler->identity(tag, image_id)) {
-                return NULL;
-            }
-        }
+    if (swf_tag_get_cid(tag) != image_id) {
+        return NULL;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, NULL);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_get_alpha_data: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, NULL) == NULL) {
+        fprintf(stderr, "swf_tag_get_alpha_data: swf_tag_create_input_detail failed\n");
         return NULL;
     }
     return swf_tag_jpeg_get_alpha_data(tag->detail, length, image_id);
@@ -377,19 +503,23 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
                           unsigned long jpeg_data_len,
                           unsigned char *alpha_data,
                           unsigned long alpha_data_len) {
-    swf_tag_info_t *tag_info;
-    swf_tag_detail_handler_t * detail_handler;
+    swf_tag_info_t *tag_info = NULL;
+    swf_tag_detail_handler_t * detail_handler = NULL;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_replace_jpeg_data: tag == NULL\n");
         return 1;
     }
+    if (jpeg_data == NULL) {
+        fprintf(stderr, "swf_tag_replace_jpeg_data: jpeg_data == NULL\n");
+        return 1;
+    }
     if (! isBitmapTag(tag->tag)) {
         return 1;
     }
     tag_info = get_swf_tag_info(tag->tag); // Bitmap Tag
     detail_handler = tag_info->detail_handler();
-    if (detail_handler->identity(tag, image_id)) {
+    if (detail_handler->get_cid(tag) !=  image_id) {
         return 1;
     }
 
@@ -428,21 +558,22 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
 
 unsigned char *
 swf_tag_get_png_data(swf_tag_t *tag, unsigned long *length, int image_id) {
-    swf_tag_info_t *tag_info;
+    swf_tag_info_t *tag_info = NULL;
     *length = 0;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_get_png_data: tag == NULL\n");
         return NULL;
     }
+    if (length == NULL) {
+        fprintf(stderr, "swf_tag_get_png_data: length == NULL\n");
+        return NULL;
+    }
     tag_info = get_swf_tag_info(tag->tag);
     if ((tag->tag != 20) && (tag->tag != 36)) {
         return NULL;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, NULL);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_get_png_data: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, NULL) == NULL) {
+        fprintf(stderr, "swf_tag_get_png_data: swf_tag_create_input_detail failed\n");
         return NULL;
     }
     return swf_tag_lossless_get_png_data(tag->detail, length, image_id, tag);
@@ -452,19 +583,23 @@ int
 swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
                          unsigned char *png_data,
                          unsigned long png_data_len) {
-    swf_tag_info_t *tag_info;
-    swf_tag_detail_handler_t *detail_handler;
+    swf_tag_info_t *tag_info = NULL;
+    swf_tag_detail_handler_t *detail_handler = NULL;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_replace_png_data: tag == NULL\n");
         return 1;
     }
+    if (png_data == NULL) {
+        fprintf(stderr, "swf_tag_replace_png_data: png_data == NULL\n");
+        return 1;
+    }
     if (! isBitmapTag(tag->tag)) {
         return 1;
     }
     tag_info = get_swf_tag_info(tag->tag); // Bitmap Tag
     detail_handler = tag_info->detail_handler();
-    if (detail_handler->identity(tag, image_id)) {
+    if (detail_handler->get_cid(tag) != image_id) {
         return 1;
     }
     
@@ -505,19 +640,23 @@ int
 swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
                          unsigned char *gif_data,
                          unsigned long gif_data_len) {
-    swf_tag_info_t *tag_info;
-    swf_tag_detail_handler_t *detail_handler;
+    swf_tag_info_t *tag_info = NULL;
+    swf_tag_detail_handler_t *detail_handler = NULL;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_replace_gif_data: tag == NULL\n");
         return 1;
     }
+    if (gif_data == NULL) {
+        fprintf(stderr, "swf_tag_replace_gif_data: gif_data == NULL\n");
+        return 1;
+    }
     if (! isBitmapTag(tag->tag)) {
         return 1;
     }
     tag_info = get_swf_tag_info(tag->tag); // Bitmap Tag
     detail_handler = tag_info->detail_handler();
-    if (detail_handler->identity(tag, image_id)) {
+    if (detail_handler->get_cid(tag) != image_id) {
         return 1;
     }
     
@@ -558,21 +697,25 @@ swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
 
 unsigned char *
 swf_tag_get_sound_data(swf_tag_t *tag, unsigned long *length, int sound_id) {
-    swf_tag_info_t *tag_info;
+    swf_tag_info_t *tag_info = NULL;
     *length = 0;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_get_sound_data: tag == NULL\n");
         return NULL;
     }
+    if (length == NULL) {
+        fprintf(stderr, "swf_tag_get_sound_data: length == NULL\n");
+        return NULL;
+    }
     tag_info = get_swf_tag_info(tag->tag);
     if (tag->tag != 14) { // DefineSound
         return NULL;
     }
     if (! tag->detail) {
-        swf_tag_create_input_detail(tag, NULL);
+        
     }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_get_sound_data: Can't create tag detail\n");
+    if (swf_tag_create_input_detail(tag, NULL) == NULL) {
+        fprintf(stderr, "swf_tag_get_sound_data: swf_tag_create_input_detail failed\n");
         return NULL;
     }
     return swf_tag_sound_get_sound_data(tag->detail, length, sound_id);
@@ -582,29 +725,23 @@ int
 swf_tag_replace_melo_data(swf_tag_t *tag, int sound_id,
                           unsigned char *melo_data,
                           unsigned long melo_data_len) {
-    swf_tag_info_t *tag_info;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_replace_melo_data: tag == NULL\n");
         return 1;
     }
+    if (melo_data == NULL) {
+        fprintf(stderr, "swf_tag_replace_melo_data: melo_data == NULL\n");
+        return 1;
+    }
     if (tag->tag != 14) { // DefineSound
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag_info && tag_info->detail_handler) {
-        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-        if (detail_handler->identity) {
-            if (detail_handler->identity(tag, sound_id)) {
-                return 1;
-            }
-        }
+    if (swf_tag_get_cid(tag) != sound_id) {
+        return 1;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, NULL);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_replace_melog_data: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, NULL) == NULL) {
+        fprintf(stderr, "swf_tag_replace_melog_data: swf_tag_create_input_detail failed\n");
         return 1;
     }
     result= swf_tag_sound_replace_melo_data(tag->detail, sound_id,
@@ -625,21 +762,21 @@ swf_tag_get_edit_string(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_get_edit_string: tag == NULL\n");
         return NULL;
     }
+    if (variable_name == NULL) {
+        fprintf(stderr, "swf_tag_get_edit_string: variable_name == NULL\n");
+        return NULL;
+    }
     if (tag->tag != 37) { // DefineEditText
         return NULL;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, swf);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_get_edit_string: swf_tag_create_input_detail failed\n");
         return NULL;
     }
     return swf_tag_edit_get_string(tag->detail,
                                    variable_name, variable_name_len);
 }
 
-
 int
 swf_tag_replace_edit_string(swf_tag_t *tag,
                             char *variable_name, int variable_name_len,
@@ -650,15 +787,15 @@ swf_tag_replace_edit_string(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_replace_edit_string: tag == NULL\n");
         return 1;
     }
+    if (variable_name == NULL) {
+        fprintf(stderr, "swf_tag_replace_edit_string: variable_name == NULL\n");
+        return 1;
+    }
     if (tag->tag != 37) { // DefineEditText
         return 1;
     }
-    
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, swf);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_replace_edit_string: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_replace_edit_string: swf_tag_create_input_detail failed\n");
         return 1;
     }
     result = swf_tag_edit_replace_string(tag->detail,
@@ -672,14 +809,12 @@ swf_tag_replace_edit_string(swf_tag_t *tag,
     return result;
 }
 
-
 int
 swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
                                   double scale_x, double scale_y,
                                   double rotate_rad,
                                   signed int trans_x, signed int trans_y,
                                   struct swf_object_ *swf) {
-    swf_tag_info_t *tag_info;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_apply_shape_matrix_factor: tag == NULL\n");
@@ -690,20 +825,11 @@ swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
         // ! DefineShape1,2,3, DefineMorphShape
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag_info && tag_info->detail_handler) {
-        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-        if (detail_handler->identity) {
-            if (detail_handler->identity(tag, shape_id)) {
-                return 1;
-            }
-        }
+    if (swf_tag_get_cid(tag) != shape_id) {
+        return 1;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, swf);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_apply_shape_matrix_factor: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_apply_shape_matrix_factor: swf_tag_create_input_detail failed\n");
         return 1;
     }
     result = swf_tag_shape_apply_matrix_factor(tag->detail, shape_id,
@@ -723,7 +849,6 @@ swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
                                   double scale_x, double scale_y,
                                   signed int trans_x, signed int trans_y,
                                   struct swf_object_ *swf) {
-    swf_tag_info_t *tag_info;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_apply_shape_rect_factor: tag == NULL\n");
@@ -734,20 +859,11 @@ swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
         // ! DefineShape1,2,3, DefineMorphShape
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag_info && tag_info->detail_handler) {
-        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-        if (detail_handler->identity) {
-            if (detail_handler->identity(tag, shape_id)) {
-                return 1;
-            }
-        }
+    if (swf_tag_get_cid(tag) != shape_id) {
+        return 1;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, swf);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_apply_shape_rect_factor: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_apply_shape_rect_factor: swf_tag_create_input_detail failed\n");
         return 1;
     }
     result = swf_tag_shape_apply_rect_factor(tag->detail, shape_id,
@@ -764,7 +880,6 @@ swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
 int
 swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
                                 struct swf_object_ *swf) {
-    swf_tag_info_t *tag_info;
     int result;
     if (tag == NULL) {
         fprintf(stderr, "swf_tag_apply_shape_type_tylled: tag == NULL\n");
@@ -775,20 +890,11 @@ swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
         // ! DefineShape1,2,3, DefineMorphShape
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag_info && tag_info->detail_handler) {
-        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
-        if (detail_handler->identity) {
-            if (detail_handler->identity(tag, shape_id)) {
-                return 1;
-            }
-        }
+    if (swf_tag_get_cid(tag) != shape_id) {
+        return 1;
     }
-    if (! tag->detail) {
-        swf_tag_create_input_detail(tag, swf);
-    }
-    if (! tag->detail) {
-        fprintf(stderr, "swf_tag_apply_shape_rect_factor: Can't create tag\n");
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_apply_shape_rect_factor: swf_tag_create_input_detail failed\n");
         return 1;
     }
     result = swf_tag_shape_apply_type_tilled(tag->detail, shape_id);
@@ -798,13 +904,17 @@ swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
         tag->length = 0;
     }
     return result;
-    
 }
 
 swf_tag_t *
 swf_tag_create_action_setvariables(y_keyvalue_t *kv) {
     int ret;
-    swf_tag_t *tag = calloc(sizeof(*tag), 1);
+    swf_tag_t *tag = NULL;
+    if (kv == NULL) {
+        fprintf(stderr, "swf_tag_create_action_setvariables: kv == NULL\n");
+        return NULL;
+    }
+    tag = calloc(sizeof(*tag), 1);
     tag->tag = 12; // DoAction
     swf_tag_info_t *tag_info = get_swf_tag_info(tag->tag);
     swf_tag_detail_handler_t *detail_handler = tag_info->detail_handler();
@@ -812,7 +922,54 @@ swf_tag_create_action_setvariables(y_keyvalue_t *kv) {
     ret = swf_tag_action_create_setvaribles(tag, kv);
     if (ret) {
         swf_tag_destroy(tag);
-        return 0;
+        return NULL;
     }
     return tag;
 }
+
+int
+swf_tag_put_action_setvariables(swf_tag_t *tag, y_keyvalue_t *kv,
+                                struct swf_object_ *swf) {
+    int ret;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_put_action_setvariables: tag == NULL\n");
+        return 1;
+    }
+    if (kv == NULL) {
+        fprintf(stderr, "swf_tag_put_action_setvariables: kv == NULL\n");
+        return 1;
+    }
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_put_action_setvariables: swf_tag_create_input_detail failed\n");
+        return 1;
+    }
+    ret = swf_tag_action_put_setvaribles(tag, kv);
+    if (ret) {
+        swf_tag_destroy(tag);
+        return 1; // NG
+    }
+    if (tag->data) {
+        free(tag->data);
+        tag->data = NULL;
+    }
+    return 0;
+}
+
+swf_tag_t *
+swf_tag_move(swf_tag_t *from_tag) {
+    swf_tag_t *to_tag = NULL;
+    if (from_tag == NULL) {
+        fprintf(stderr, "swf_tag_move: from_tag == NULL\n");
+        return NULL;
+    }
+    to_tag = calloc(sizeof(*to_tag), 1);
+    to_tag->tag = from_tag->tag;
+    to_tag->length = from_tag->length;
+    to_tag->length_longformat = from_tag->length_longformat;
+    to_tag->data = from_tag->data;
+    from_tag->data = NULL;
+    from_tag->length = 0;
+    to_tag->detail = from_tag->detail;
+    from_tag->detail = NULL;
+    return to_tag;
+}
