@@ -26,7 +26,8 @@ typedef struct swf_tag_ {
 typedef struct swf_tag_detail_handler_ {
     void          * (*create)   (void);
     int             (*input)    (swf_tag_t *tag, struct swf_object_ *swf);
-    int             (*identity) (swf_tag_t *tag, int id);
+    int             (*get_cid)  (swf_tag_t *tag);
+    int             (*replace_cid) (swf_tag_t *tag, int cid);
     unsigned char * (*output)   (swf_tag_t *tag, unsigned long *length,
                                  struct swf_object_ *swf);
     void            (*print)    (swf_tag_t *tag, struct swf_object_ *swf,
@@ -46,8 +47,14 @@ extern swf_tag_t *swf_tag_create(bitstream_t *bs);
 extern void swf_tag_destroy(swf_tag_t *tag);
 extern int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *swf);
 extern void swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf, int indent_depth);
+extern void *swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf);
+extern void swf_tag_destroy_detail(swf_tag_t *tag);
 
-extern int swf_tag_identity(swf_tag_t *tag, int cid);
+extern int swf_tag_get_cid(swf_tag_t *tag);
+extern int swf_tag_replace_cid(swf_tag_t *tag, int cid);
+
+extern int swf_tag_get_refcid(swf_tag_t *tag);
+extern int swf_tag_replace_refcid(swf_tag_t *tag, int cid);
 
 /* image */
 
@@ -89,8 +96,6 @@ extern int swf_tag_replace_edit_string(swf_tag_t *tag,
                                        int initial_text_len,
                                        struct swf_object_ *swf);
 
-extern int swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf);
-
 extern int swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
                                              double scale_x, double scale_y,
                                              double rotate_rad,
@@ -106,5 +111,10 @@ extern int swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
                                            struct swf_object_ *swf);
 
 extern swf_tag_t *swf_tag_create_action_setvariables(y_keyvalue_t *kv);
+extern int swf_tag_put_action_setvariables(swf_tag_t *tag,
+                                           y_keyvalue_t *kv,
+                                           struct swf_object_ *swf);
+
+extern swf_tag_t * swf_tag_move(swf_tag_t *tag);
 
 #endif /* __SWF_TAG_H__ */
