@@ -16,7 +16,8 @@ swf_tag_detail_handler_t *
 swf_tag_action_detail_handler(void) {
     action_detail_handler.create   = swf_tag_action_create_detail;
     action_detail_handler.input    = swf_tag_action_input_detail;
-    action_detail_handler.identity = swf_tag_action_identity_detail;
+    action_detail_handler.get_cid  = NULL;
+    action_detail_handler.replace_cid  = NULL;
     action_detail_handler.output   = swf_tag_action_output_detail;
     action_detail_handler.print    = swf_tag_action_print_detail;
     action_detail_handler.destroy  = swf_tag_action_destroy_detail;
@@ -66,12 +67,6 @@ swf_tag_action_input_detail(swf_tag_t *tag, struct swf_object_ *swf) {
     return 0;
 }
 
-int swf_tag_action_identity_detail(swf_tag_t *tag, int id) {
-    (void) tag;
-    (void) id;
-    return 1;
-}
-
 unsigned char *
 swf_tag_action_output_detail(swf_tag_t *tag, unsigned long *length,
                              struct swf_object_ *swf) {
@@ -145,16 +140,54 @@ swf_tag_action_create_setvaribles(swf_tag_t *tag, y_keyvalue_t *kv) {
         bitstream_putbyte(bs, 0x96); // Push Data
         bitstream_putbytesLE(bs, key_len + 2 , 2);
         bitstream_putbyte(bs, 0);
-        bitstream_putstring(bs, key, key_len);
+        bitstream_putstring(bs, (unsigned char*) key, key_len);
         bitstream_putbyte(bs, 0);
         bitstream_putbyte(bs, 0x96); // Push Data
         bitstream_putbytesLE(bs, value_len + 2 , 2);
         bitstream_putbyte(bs, 0);
-        bitstream_putstring(bs, value, value_len);
+        bitstream_putstring(bs, (unsigned char*) value, value_len);
         bitstream_putbyte(bs, 0);
         bitstream_putbyte(bs, 0x1d); // Set Variable
     }
     bitstream_putbyte(bs, 0); // End
+    if (swf_tag_action->action_record) {
+        free(swf_tag_action->action_record);
+    }
+    swf_tag_action->action_record = bitstream_steal(bs, &data_len);
+    swf_tag_action->action_record_len = data_len;
+    bitstream_close(bs);
+    return 0;
+}
+
+int
+swf_tag_action_put_setvaribles(swf_tag_t *tag, y_keyvalue_t *kv) {
+    bitstream_t *bs;
+    char *key, *value;
+    int key_len, value_len;
+    unsigned long data_len;
+    swf_tag_action_detail_t *swf_tag_action = (swf_tag_action_detail_t *) tag->detail;
+    swf_tag_action->action_sprite = 0;
+    bs = bitstream_open();
+    y_keyvalue_rewind(kv);
+    while (y_keyvalue_next(kv)) {
+        key   = y_keyvalue_get_currentkey(kv, &key_len);
+        value = y_keyvalue_get_currentvalue(kv, &value_len);
+        bitstream_putbyte(bs, 0x96); // Push Data
+        bitstream_putbytesLE(bs, key_len + 2 , 2);
+        bitstream_putbyte(bs, 0);
+        bitstream_putstring(bs, (unsigned char *)key, key_len);
+        bitstream_putbyte(bs, 0);
+        bitstream_putbyte(bs, 0x96); // Push Data
+        bitstream_putbytesLE(bs, value_len + 2 , 2);
+        bitstream_putbyte(bs, 0);
+        bitstream_putstring(bs, (unsigned char *)value, value_len);
+        bitstream_putbyte(bs, 0);
+        bitstream_putbyte(bs, 0x1d); // Set Variable
+    }
+    if (swf_tag_action->action_record) {
+        bitstream_putstring(bs, swf_tag_action->action_record, swf_tag_action->action_record_len);
+        free(swf_tag_action->action_record);
+    }
     swf_tag_action->action_record = bitstream_steal(bs, &data_len);
     swf_tag_action->action_record_len = data_len;
     bitstream_close(bs);
